#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>

#include <TSystem.h>

#include "CalibrationManager.hh"

using namespace std;


void CalibrationManager::ReadCalibration(){
  if(detector_name == "ppac"){
    cout << file_name << endl;
  }else{
    ifstream fin(file_name.Data());
    string buf;

    Int_t flag = 0;
    while(getline(fin, buf)){
      if(buf[0]=='#' || buf[0]=='c' || buf == ""){ continue; }
      if(flag == 0){
        dim = stoi(buf);
        flag++;
        continue;
      }

      if((buf.find(';') != buf.rfind(';')) || buf.find(';') == string::npos){
        cerr << "ERROR: calibration file is not correct" << endl;
        exit(0);
      }
      string name = buf.substr(buf.find(';')+1);
      if(name[0] == ' '){ name.erase(0, 1); }
      if(name[name.length()-1] == ' '){ name.pop_back(); }
      replace(name.begin(), name.end(), ' ', '_');
      replace(name.begin(), name.end(), '/', '_');
      replace(name.begin(), name.end(), '-', '_');

      branch_name.push_back(name);

      vector<Double_t> calib;
      Int_t check_dim = 0;
      string tmp = buf.substr(0, buf.find(';'));
      stringstream ss{tmp};
      string val;
      while(getline(ss, val, ',')){
        while(val[0] == ' '){ val.erase(0, 1); }
        while(val[val.length()-1] == ' '){ val.pop_back(); }
        double d_val = stod(val);
        calib.push_back(d_val);
        check_dim++;
      }
      if(dim != check_dim-1){
        cerr << "ERROR: dimention miss in parameter file" << endl;
        exit(0);
      }
      parameters.push_back(calib);
    }
    fin.close();
  }
}


void CalibrationManager::ReadMUXposCalib(TString pos_file_name){
  ifstream fin(pos_file_name.Data());
  string buf;

  while(getline(fin, buf)){
    if(buf[0]=='#' || buf[0]=='c' || buf == ""){ continue; }
    vector<Double_t> mpos_peak;
    string pos_array = buf.substr(0, buf.find(';'));
    stringstream ss{pos_array};
    string val;
    while(getline(ss, val, ',')){
      if(val == "" || val == " "){ continue; }
      while(val[0] == ' '){ val.erase(0, 1); }
      while(val[val.length()-1] == ' '){ val.pop_back(); }
      double d_val = stod(val);
      mpos_peak.push_back(d_val);
    }
    if(mpos_peak.size() != 16){
      cerr << "ERROR: MUX position calibration file is not correct" << endl;
      exit(0);
    }
    vector<Double_t> mpos_tmp;
    mpos_tmp.push_back(mpos_peak[0] - (mpos_peak[1]-mpos_peak[0])/2.0);
    for(Int_t i=0; i<15; i++){ mpos_tmp.push_back((mpos_peak[i]+mpos_peak[i+1])/2.0); }
    mpos_tmp.push_back(mpos_peak[15] + (mpos_peak[15]-mpos_peak[14])/2.0);

    position_prms.push_back(mpos_tmp);
    if(mpos_tmp.size() != 17){
      cerr << "ERROR: MUX position calibration file is not correct" << endl;
      exit(0);
    }
  }
  fin.close();
}
